@@ -8,36 +8,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #-------------------------------------------------------------------------------
 
-#' Apply the dictionary to a tibble (deprecated)
-#'
-#' Deprecated: use \link{dictionary.apply} instead.
-#'
-#' @param tibble Tibble to be decorated.
-#' @param variables A data frame with one row per variable (column name) and then one column per property/attribute.
-#' @param categories A data frame with one row per category (columns variable and name) and then column per property/attribute.
-#' @examples 
-#' \dontrun{
-#' data <- tibble::as_tibble(mtcars)
-#' variables <- tibble::tribble(
-#'   ~name, ~valueType, ~`label:en`,  ~`Namespace::Name`, ~unit, ~repeatable, ~index,
-#'   "mpg", "decimal", "Mpg label",  "Value1", "years", 0, 1,
-#'   "cyl", "decimal", "Cyl label",  "Value2", "kg/m2", 0, 2,
-#'   "disp", "decimal", "Disp label", NA, NA, 1, 3
-#' )
-#' categories <- tibble::tribble(
-#'   ~variable, ~name, ~missing, ~`label:en`, ~`label:fr`,
-#'   "cyl", "4", 0, "Four", "Quatre",
-#'   "cyl", "6", 0, "Six", "Six",
-#'   "cyl", "8", 1, "Height", "Huit"
-#' )
-#' data <- dictionary.apply(data, variables, categories)
-#' }
-#' @export
-harmo.dictionary_apply <- function(tibble, variables, categories = NULL) {
-  warning("Deprecated: harmo.dictionary_apply() is replaced by dictionary.apply()")
-  dictionary.apply(tibble, variables, categories = categories)
-}
-
 #' Apply the dictionary to a tibble
 #'
 #' Apply the dictionary described in a Opal Excel format as attributes of the tibble's columns.
@@ -168,36 +138,62 @@ dictionary.apply <- function(tibble, variables, categories = NULL) {
   tbl
 }
 
-#' Update the dictionary of a Opal table (deprecated)
-#' 
-#' Deprecated: use \link{opal.table_dictionary_update} instead.
-#' 
-#' @param opal Opal connection object.
-#' @param project Project name where the table will be located.
-#' @param table Destination table name.
-#' @param variables A data frame with one row per variable (column name) and then one column per property/attribute (Opal Excel format).
-#' @param categories A data frame with one row per category (columns variable and name) and then column per property/attribute (Opal Excel format). If there are
-#' no categories, this parameter is optional.
+#' Inspect the dictionary of a tibble
+#'
+#' Inspect the data dictionary, checking for inconsistencies. Error is raised when the identifiers
+#' column cannot be found.
+#'
+#' @param tibble Tibble to be inspected.
+#' @param id.name The name of the column representing the entity identifiers. Default is 'id'.
+#' @param warn Print warning messages. Default is TRUE.
+#' @return TRUE if inspection is successful, FALSE otherwise.
 #' @examples 
 #' \dontrun{
-#' o <- opal.login('administrator','password', url='https://opal-demo.obiba.org')
-#' variables <- tibble::tribble(
-#'   ~name, ~valueType, ~`label:en`,  ~`Namespace::Name`, ~unit, ~repeatable, ~index,
-#'   "mpg", "decimal", "Mpg label",  "Value1", "years", 0, 1,
-#'   "cyl", "decimal", "Cyl label",  "Value2", "kg/m2", 0, 2,
-#'   "disp", "decimal", "Disp label", NA, NA, 1, 3
-#' )
-#' categories <- tibble::tribble(
-#'   ~variable, ~name, ~missing, ~`label:en`, ~`label:fr`,
-#'   "cyl", "4", 0, "Four", "Quatre",
-#'   "cyl", "6", 0, "Six", "Six",
-#'   "cyl", "8", 1, "Height", "Huit"
-#' )
-#' opal.table_dictionary_update(o, "test", "mtcars", variables, categories)
+#' # prepare datasets of visits, several visits for one patient
+#' visits <- tibble::tribble(
+#' ~id, ~patient_id, ~sex, ~visit_date,
+#' 1, 1, "M", as.Date("2020-01-01"),
+#' 2, 2, "F", as.Date("2020-01-02"),
+#' 3, 3, "M", as.Date("2020-01-03"),
+#' 4, 3, "M", as.Date("2020-01-04"))
+#' 
+#' o <- opal.login("administrator", "password", url = "https://opal-demo.obiba.org")
+#' 
+#' # save visits dataset
+#' opal.table_save(o, visits, "RSRC", "visits", type = "Visit", force = TRUE)
+#' 
+#' # get visit and make it a dataset of patients 
+#' patients <- opal.table_get(o, "RSRC", "visits", id.name = "id")
+#' # set dataset IDs
+#' patients$visit_id <- patients$id
+#' patients$id <- patients$patient_id
+#' patients$patient_id <- NULL
+#' patients
+#' 
+#' # save patients dataset, there should be a warning that some variables are not repeatable
+#' # while there are patients with multiple data lines
+#' opal.table_save(o, patients, "RSRC", "patients", type = "Participant", force = TRUE)
+#' 
 #' opal.logout(o)
 #' }
 #' @export
-harmo.dictionary_update <- function(opal, project, table, variables, categories = NULL) {
-  warning("Deprecated: harmo.dictionary_update() is deprecated by opal.table_dictionary_update()")
-  opal.table_dictionary_update(opal, project, table, variables, categories = categories)
+dictionary.inspect <- function(tibble, id.name = 'id', warn = TRUE) {
+  # check for ID column, mandatory
+  if (!(id.name %in% colnames(tibble))) {
+    stop("No identifiers column with name '", id.name, "'", call. = FALSE)
+  }
+  rval <- TRUE
+  # check whether it is a multilines dataset
+  if (length(unique(tibble[[id.name]]) < nrow(tibble))) {
+    for (n in colnames(tibble)) {
+      if (n != id.name) {
+        attrs <- attributes(tibble[[n]])
+        if (!is.null(attrs$opal.repeatable) && attrs$opal.repeatable == 0) {
+          rval <- FALSE
+          warning("Column is not repeatable in a multilines dataset (identifiers column is '", id.name,"'): ", n, call. = FALSE, immediate. = TRUE)
+        }
+      }
+    }
+  }
+  rval
 }
