@@ -92,7 +92,7 @@ opal.logout <- function(opal, save=FALSE) {
     res <- try(.rmOpalSession(opal), silent = TRUE)
     opal$sid <- NULL
   }
-  if (.is.verbose() && !is.null(res) && length(res) > 0) {
+  if (!is.null(res) && length(res) > 0) {
     return(res)
   }
 }
@@ -156,6 +156,7 @@ opal.version_compare <- function(opal, version) {
 #' @param opal Opal object.
 #' @param ... Resource path segments.
 #' @param query Named list of query parameters.
+#' @param acceptType The type of the body content. Default is 'application/json', i.e. a serialized R object or an error message.
 #' @param callback A callback function to handle the response object.
 #' @import httr
 #' @examples 
@@ -165,8 +166,8 @@ opal.version_compare <- function(opal, version) {
 #' opal.logout(o)
 #' }
 #' @export
-opal.get <- function(opal, ..., query=list(), callback=NULL) {
-  r <- GET(.url(opal, ...), query=query, config=opal$config, handle = opal$handle, .verbose())
+opal.get <- function(opal, ..., query=list(), acceptType='application/json', callback=NULL) {
+  r <- GET(.url(opal, ...), query=query, accept(acceptType), config=opal$config, handle = opal$handle, .verbose())
   .handleResponseOrCallback(opal, r, callback)
 }
 
@@ -177,7 +178,8 @@ opal.get <- function(opal, ..., query=list(), callback=NULL) {
 #' @param ... Resource path segments.
 #' @param query Named list of query parameters.
 #' @param body The body of the request.
-#' @param contentType The type of the body content.
+#' @param contentType The type of the body content. Default is 'application/x-rscript'.
+#' @param acceptType The type of the body content. Default is 'application/json', i.e. a serialized R object or an error message.
 #' @param callback A callback function to handle the response object.
 #' @import httr
 #' @examples 
@@ -187,8 +189,8 @@ opal.get <- function(opal, ..., query=list(), callback=NULL) {
 #' opal.logout(o)
 #' }
 #' @export
-opal.post <- function(opal, ..., query=list(), body='', contentType='application/x-rscript', callback=NULL) {
-  r <- POST(.url(opal, ...), query=query, body=body, content_type(contentType), config=opal$config, handle = opal$handle, .verbose())
+opal.post <- function(opal, ..., query=list(), body='', contentType='application/x-rscript', acceptType='application/json', callback=NULL) {
+  r <- POST(.url(opal, ...), query=query, body=body, content_type(contentType), accept(acceptType), config=opal$config, handle = opal$handle, .verbose())
   .handleResponseOrCallback(opal, r, callback)
 }
 
@@ -297,15 +299,19 @@ opal.delete <- function(opal, ..., query=list(), callback=NULL) {
 .handleError <- function(opal, response) {
   headers <- httr::headers(response)
   content <- .getContent(opal, response)
-  msg <- http_status(response)$message
+  if (.is.verbose()) {
+    warning(httr::content(response, as="text", encoding = opal$encoding), call. = FALSE)
+  }
+  msg <- paste0("[", http_status(response)$message, "]")
   if (is.null(content)) {
     stop(msg, call.=FALSE)
   }
   
   if ("status" %in% names(content)) {
-    msg <- paste0(msg, "; ", content$status)
     if ("arguments" %in% names(content)) {
-      msg <- paste0(msg, ": ", paste(content$arguments, collapse = ", "))
+      msg <- paste0(msg, " ", paste(content$arguments, collapse = ", "))
+    } else {
+      msg <- paste0(msg, " ", content$status)
     }
     stop(msg, call.=FALSE)
   }
